@@ -132,12 +132,28 @@ clutter_animation_real_completed (ClutterAnimation *self)
 {
   ClutterAnimationPrivate *priv = self->priv;
   ClutterAnimation *animation;
+  gpointer key, value;
+  GHashTableIter iter;
+
+  /* explicitly set the final state of the animation */
+  g_hash_table_iter_init (&iter, priv->properties);
+  while (g_hash_table_iter_next (&iter, &key, &value))
+    {
+      const gchar *p_name = key;
+      ClutterInterval *interval = value;
+      GValue *p_value;
+
+      p_value = clutter_interval_peek_final_value (interval);
+      g_object_set_property (priv->object, p_name, p_value);
+    }
 
   /* at this point, if this animation was created by clutter_actor_animate()
    * and friends, the animation will be attached to the object's data; since
    * we want to allow developers to use g_signal_connect_after("completed")
    * to concatenate a new animation, we need to remove the animation back
-   * pointer here, and unref() the animation
+   * pointer here, and unref() the animation. FIXME - we might want to
+   * provide a clutter_animation_attach()/clutter_animation_detach() pair
+   * to let the user reattach an animation
    */
   animation = g_object_get_qdata (priv->object, quark_object_animation);
   if (animation == self)
@@ -1306,7 +1322,7 @@ clutter_animation_set_alpha (ClutterAnimation *animation,
 
   priv->alpha = g_object_ref_sink (alpha);
   priv->alpha_notify_id =
-    g_signal_connect (priv->alpha, "notify::value",
+    g_signal_connect (priv->alpha, "notify::alpha",
                       G_CALLBACK (on_alpha_notify),
                       animation);
 
@@ -1947,8 +1963,8 @@ clutter_actor_animate (ClutterActor *actor,
  * @mode: an animation mode logical id
  * @duration: duration of the animation, in milliseconds
  * @n_properties: number of property names and values
- * @properties: (array length=n_properties): a vector containing the
- *    property names to set
+ * @properties: (array length=n_properties) (element-type utf8): a vector
+ *    containing the property names to set
  * @values: (array length=n_properies): a vector containing the
  *    property values to set
  *
@@ -1999,8 +2015,8 @@ clutter_actor_animatev (ClutterActor        *actor,
  * @mode: an animation mode logical id
  * @timeline: a #ClutterTimeline
  * @n_properties: number of property names and values
- * @properties: (array length=n_properties): a vector containing the
- *    property names to set
+ * @properties: (array length=n_properties) (element-type utf8): a vector
+ *    containing the property names to set
  * @values: (array length=n_properies): a vector containing the
  *    property values to set
  *
@@ -2054,8 +2070,8 @@ clutter_actor_animate_with_timelinev (ClutterActor        *actor,
  * @actor: a #ClutterActor
  * @alpha: a #ClutterAlpha
  * @n_properties: number of property names and values
- * @properties: (array length=n_properties): a vector containing the
- *    property names to set
+ * @properties: (array length=n_properties) (element-type utf8): a vector
+ *    containing the property names to set
  * @values: (array length=n_properies): a vector containing the
  *    property values to set
  *
