@@ -61,8 +61,6 @@ enum
   LAST_SIGNAL
 };
 
-static guint group_signals[LAST_SIGNAL] = { 0 };
-
 static void clutter_container_iface_init (ClutterContainerIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (ClutterGroup,
@@ -98,8 +96,7 @@ clutter_group_paint (ClutterActor *actor)
 
       g_assert (child != NULL);
 
-      if (CLUTTER_ACTOR_IS_VISIBLE (child))
-	clutter_actor_paint (child);
+      clutter_actor_paint (child);
     }
 
   CLUTTER_NOTE (PAINT, "ClutterGroup paint leave '%s'",
@@ -107,44 +104,36 @@ clutter_group_paint (ClutterActor *actor)
                                                : "unknown");
 }
 
-static void
-clutter_group_realize (ClutterActor *actor)
-{
-  clutter_container_foreach (CLUTTER_CONTAINER (actor),
-                             CLUTTER_CALLBACK (clutter_actor_realize),
-                             NULL);
-}
-
-static void
-clutter_group_unrealize (ClutterActor *actor)
-{
-  clutter_container_foreach (CLUTTER_CONTAINER (actor),
-                             CLUTTER_CALLBACK (clutter_actor_unrealize),
-                             NULL);
-}
-
 static void
 clutter_group_pick (ClutterActor       *actor,
 		    const ClutterColor *color)
 {
+  ClutterGroupPrivate *priv = CLUTTER_GROUP (actor)->priv;
+  GList               *child_item;
+
   /* Chain up so we get a bounding box pained (if we are reactive) */
   CLUTTER_ACTOR_CLASS (clutter_group_parent_class)->pick (actor, color);
 
-  /* Just forward to the paint call which in turn will trigger
-   * the child actors also getting 'picked'.
-   */
-  if (CLUTTER_ACTOR_IS_VISIBLE (actor))
-    clutter_group_paint (actor);
+  for (child_item = priv->children;
+       child_item != NULL;
+       child_item = child_item->next)
+    {
+      ClutterActor *child = child_item->data;
+
+      g_assert (child != NULL);
+
+      clutter_actor_paint (child);
+    }
 }
 
 static void
-clutter_fixed_layout_get_preferred_width (GList       *children,
-                                          ClutterUnit *min_width_p,
-                                          ClutterUnit *natural_width_p)
+clutter_fixed_layout_get_preferred_width (GList  *children,
+                                          gfloat *min_width_p,
+                                          gfloat *natural_width_p)
 {
   GList *l;
-  ClutterUnit min_left, min_right;
-  ClutterUnit natural_left, natural_right;
+  gdouble min_left, min_right;
+  gdouble natural_left, natural_right;
 
   min_left = 0;
   min_right = 0;
@@ -154,9 +143,9 @@ clutter_fixed_layout_get_preferred_width (GList       *children,
   for (l = children; l != NULL; l = l->next)
     {
       ClutterActor *child = l->data;
-      ClutterUnit child_x, child_min, child_natural;
+      gfloat child_x, child_min, child_natural;
 
-      child_x = clutter_actor_get_xu (child);
+      child_x = clutter_actor_get_x (child);
 
       clutter_actor_get_preferred_size (child,
                                         &child_min, NULL,
@@ -214,13 +203,13 @@ clutter_fixed_layout_get_preferred_width (GList       *children,
 }
 
 static void
-clutter_fixed_layout_get_preferred_height (GList       *children,
-                                           ClutterUnit *min_height_p,
-                                           ClutterUnit *natural_height_p)
+clutter_fixed_layout_get_preferred_height (GList  *children,
+                                           gfloat *min_height_p,
+                                           gfloat *natural_height_p)
 {
   GList *l;
-  ClutterUnit min_top, min_bottom;
-  ClutterUnit natural_top, natural_bottom;
+  gdouble min_top, min_bottom;
+  gdouble natural_top, natural_bottom;
 
   min_top = 0;
   min_bottom = 0;
@@ -230,9 +219,9 @@ clutter_fixed_layout_get_preferred_height (GList       *children,
   for (l = children; l != NULL; l = l->next)
     {
       ClutterActor *child = l->data;
-      ClutterUnit child_y, child_min, child_natural;
+      gfloat child_y, child_min, child_natural;
 
-      child_y = clutter_actor_get_yu (child);
+      child_y = clutter_actor_get_y (child);
 
       clutter_actor_get_preferred_size (child,
                                         NULL, &child_min,
@@ -290,23 +279,23 @@ clutter_fixed_layout_get_preferred_height (GList       *children,
 }
 
 static void
-clutter_fixed_layout_allocate (GList    *children,
-                               gboolean  absolute_origin_changed)
+clutter_fixed_layout_allocate (GList                  *children,
+                               ClutterAllocationFlags  flags)
 {
   GList *l;
 
   for (l = children; l != NULL; l = l->next)
     {
       ClutterActor *child = l->data;
-      clutter_actor_allocate_preferred_size (child, absolute_origin_changed);
+      clutter_actor_allocate_preferred_size (child, flags);
     }
 }
 
 static void
 clutter_group_get_preferred_width (ClutterActor *self,
-                                   ClutterUnit   for_height,
-                                   ClutterUnit  *min_width_p,
-                                   ClutterUnit  *natural_width_p)
+                                   gfloat        for_height,
+                                   gfloat       *min_width_p,
+                                   gfloat       *natural_width_p)
 {
   ClutterGroupPrivate *priv = CLUTTER_GROUP (self)->priv;
 
@@ -318,9 +307,9 @@ clutter_group_get_preferred_width (ClutterActor *self,
 
 static void
 clutter_group_get_preferred_height (ClutterActor *self,
-                                    ClutterUnit   for_width,
-                                    ClutterUnit  *min_height_p,
-                                    ClutterUnit  *natural_height_p)
+                                    gfloat        for_width,
+                                    gfloat       *min_height_p,
+                                    gfloat       *natural_height_p)
 {
   ClutterGroupPrivate *priv = CLUTTER_GROUP (self)->priv;
 
@@ -331,22 +320,21 @@ clutter_group_get_preferred_height (ClutterActor *self,
 }
 
 static void
-clutter_group_allocate (ClutterActor          *self,
-                        const ClutterActorBox *box,
-                        gboolean               origin_changed)
+clutter_group_allocate (ClutterActor           *self,
+                        const ClutterActorBox  *box,
+                        ClutterAllocationFlags  flags)
 {
   ClutterGroupPrivate *priv = CLUTTER_GROUP (self)->priv;
 
   /* chain up to set actor->allocation */
-  CLUTTER_ACTOR_CLASS (clutter_group_parent_class)->allocate (self, box,
-                                                              origin_changed);
+  CLUTTER_ACTOR_CLASS (clutter_group_parent_class)->allocate (self, box, flags);
 
   /* Note that fixed-layout allocation of children does not care what
    * allocation the container received, so "box" is not passed in
    * here. We do not require that children's allocations are completely
    * contained by our own.
    */
-  clutter_fixed_layout_allocate (priv->children, origin_changed);
+  clutter_fixed_layout_allocate (priv->children, flags);
 }
 
 static void
@@ -393,13 +381,6 @@ clutter_group_real_add (ClutterContainer *container,
 
   g_object_ref (actor);
 
-  /* the old ClutterGroup::add signal was emitted before the
-   * actor was added to the group, so that the class handler
-   * would actually add it. we need to emit the ::add signal
-   * here so that handlers expecting it will not freak out.
-   */
-  g_signal_emit (group, group_signals[ADD], 0, actor);
-
   priv->children = g_list_append (priv->children, actor);
   clutter_actor_set_parent (actor, CLUTTER_ACTOR (group));
 
@@ -410,7 +391,7 @@ clutter_group_real_add (ClutterContainer *container,
 
   g_signal_emit_by_name (container, "actor-added", actor);
 
-  clutter_group_sort_depth_order (group);
+  clutter_container_sort_depth_order (container);
 
   g_object_unref (actor);
 }
@@ -424,13 +405,6 @@ clutter_group_real_remove (ClutterContainer *container,
 
   g_object_ref (actor);
 
-  /* the old ClutterGroup::remove signal was emitted before the
-   * actor was removed from the group. see the comment in
-   * clutter_group_real_add() above for why we need to emit ::remove
-   * here and not later
-   */
-  g_signal_emit (group, group_signals[REMOVE], 0, actor);
-
   priv->children = g_list_remove (priv->children, actor);
   clutter_actor_unparent (actor);
 
@@ -595,61 +569,18 @@ clutter_group_class_init (ClutterGroupClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
 
+  g_type_class_add_private (object_class, sizeof (ClutterGroupPrivate));
+
   object_class->dispose = clutter_group_dispose;
 
   actor_class->paint           = clutter_group_paint;
   actor_class->pick            = clutter_group_pick;
   actor_class->show_all        = clutter_group_real_show_all;
   actor_class->hide_all        = clutter_group_real_hide_all;
-  actor_class->realize         = clutter_group_realize;
-  actor_class->unrealize       = clutter_group_unrealize;
 
   actor_class->get_preferred_width  = clutter_group_get_preferred_width;
   actor_class->get_preferred_height = clutter_group_get_preferred_height;
   actor_class->allocate             = clutter_group_allocate;
-
-  /**
-   * ClutterGroup::add:
-   * @group: the #ClutterGroup that received the signal
-   * @actor: the actor added to the group
-   *
-   * The ::add signal is emitted each time an actor has been added
-   * to the group.
-   *
-   * @Deprecated: 0.4: This signal is deprecated, you should connect
-   *   to the ClutterContainer::actor-added signal instead.
-   */
-  group_signals[ADD] =
-    g_signal_new ("add",
-		  G_OBJECT_CLASS_TYPE (object_class),
-		  G_SIGNAL_RUN_FIRST,
-		  G_STRUCT_OFFSET (ClutterGroupClass, add),
-		  NULL, NULL,
-		  clutter_marshal_VOID__OBJECT,
-		  G_TYPE_NONE, 1,
-		  CLUTTER_TYPE_ACTOR);
-  /**
-   * ClutterGroup::remove:
-   * @group: the #ClutterGroup that received the signal
-   * @actor: the actor added to the group
-   *
-   * The ::remove signal is emitted each time an actor has been removed
-   * from the group
-   *
-   * @Deprecated: 0.4: This signal is deprecated, you should connect
-   *   to the ClutterContainer::actor-removed signal instead
-   */
-  group_signals[REMOVE] =
-    g_signal_new ("remove",
-		  G_OBJECT_CLASS_TYPE (object_class),
-		  G_SIGNAL_RUN_FIRST,
-		  G_STRUCT_OFFSET (ClutterGroupClass, remove),
-		  NULL, NULL,
-		  clutter_marshal_VOID__OBJECT,
-		  G_TYPE_NONE, 1,
-		  CLUTTER_TYPE_ACTOR);
-
-  g_type_class_add_private (object_class, sizeof (ClutterGroupPrivate));
 }
 
 static void
@@ -671,67 +602,6 @@ clutter_group_new (void)
   return g_object_new (CLUTTER_TYPE_GROUP, NULL);
 }
 
-/**
- * clutter_group_add_many_valist:
- * @group: a #ClutterGroup
- * @first_actor: the #ClutterActor actor to add to the group
- * @var_args: the actors to be added
- *
- * Similar to clutter_group_add_many() but using a va_list.  Use this
- * function inside bindings.
- *
- * @Deprecated: 0.4: This function is obsolete, use
- *   clutter_container_add_valist() instead.
- */
-void
-clutter_group_add_many_valist (ClutterGroup *group,
-			       ClutterActor *first_actor,
-			       va_list       var_args)
-{
-  clutter_container_add_valist (CLUTTER_CONTAINER (group), first_actor, var_args);
-}
-
-/**
- * clutter_group_add_many:
- * @group: A #ClutterGroup
- * @first_actor: the #ClutterActor actor to add to the group
- * @Varargs: additional actors to add to the group
- *
- * Adds a %NULL-terminated list of actors to a group.  This function is
- * equivalent to calling clutter_group_add() for each member of the list.
- *
- * @Deprecated: 0.4: This function is obsolete, use clutter_container_add()
- *   instead.
- */
-void
-clutter_group_add_many (ClutterGroup *group,
-		        ClutterActor *first_actor,
-			...)
-{
-  va_list args;
-
-  va_start (args, first_actor);
-  clutter_container_add_valist (CLUTTER_CONTAINER (group), first_actor, args);
-  va_end (args);
-}
-
-/**
- * clutter_group_remove
- * @group: A #ClutterGroup
- * @actor: A #ClutterActor
- *
- * Removes a child #ClutterActor from the parent #ClutterGroup.
- *
- * @Deprecated: 0.4: This function is obsolete, use
- *   clutter_container_remove_actor() instead.
- */
-void
-clutter_group_remove (ClutterGroup *group,
-		      ClutterActor *actor)
-{
-  clutter_container_remove_actor (CLUTTER_CONTAINER (group), actor);
-}
-
 /**
  * clutter_group_remove_all:
  * @group: A #ClutterGroup
@@ -755,24 +625,6 @@ clutter_group_remove_all (ClutterGroup *group)
     }
 }
 
-/**
- * clutter_group_get_children:
- * @self: A #ClutterGroup
- *
- * Get a list containing all actors contained in the group.
- *
- * Return value: A list of #ClutterActors. You  should free the returned
- *   list using g_list_free() when finished using it.
- *
- * @Deprecated: 0.4: This function is obsolete, use
- *   clutter_container_get_children() instead.
- */
-GList*
-clutter_group_get_children (ClutterGroup *self)
-{
-  return clutter_container_get_children (CLUTTER_CONTAINER (self));
-}
-
 /**
  * clutter_group_get_n_children:
  * @self: A #ClutterGroup
@@ -782,7 +634,7 @@ clutter_group_get_children (ClutterGroup *self)
  * Return value: The number of child actors held in the group.
  *
  * Since: 0.2
- **/
+ */
 gint
 clutter_group_get_n_children (ClutterGroup *self)
 {
@@ -798,10 +650,11 @@ clutter_group_get_n_children (ClutterGroup *self)
  *
  * Gets a groups child held at @index_ in stack.
  *
- * Return value: (transfer none): A Clutter actor or NULL if @index_ is invalid.
+ * Return value: (transfer none): A Clutter actor, or %NULL if
+ *   @index_ is invalid.
  *
  * Since: 0.2
- **/
+ */
 ClutterActor *
 clutter_group_get_nth_child (ClutterGroup *self,
 			     gint          index_)
@@ -810,70 +663,3 @@ clutter_group_get_nth_child (ClutterGroup *self,
 
   return g_list_nth_data (self->priv->children, index_);
 }
-
-/**
- * clutter_group_raise:
- * @self: a #ClutterGroup
- * @actor: a #ClutterActor
- * @sibling: a #ClutterActor
- *
- * Raises @actor to @sibling level in the depth ordering.
- *
- * Deprecated: 0.6: Use clutter_container_raise_child() instead.
- */
-void
-clutter_group_raise (ClutterGroup *self,
-		     ClutterActor *actor,
-		     ClutterActor *sibling)
-{
-  g_return_if_fail (CLUTTER_IS_GROUP (self));
-  g_return_if_fail (CLUTTER_IS_ACTOR (actor));
-  g_return_if_fail (sibling == NULL || CLUTTER_IS_ACTOR (sibling));
-
-  if (actor == sibling)
-    return;
-
-  clutter_container_raise_child (CLUTTER_CONTAINER (self), actor, sibling);
-}
-
-/**
- * clutter_group_lower:
- * @self: a #ClutterGroup
- * @actor: a #ClutterActor
- * @sibling: a #ClutterActor
- *
- * Lowers @actor to @sibling level in the depth ordering.
- *
- * Deprecated: 0.6: Use clutter_container_lower_child() instead
- */
-void
-clutter_group_lower (ClutterGroup *self,
-		     ClutterActor *actor,
-		     ClutterActor *sibling)
-{
-  g_return_if_fail (CLUTTER_IS_GROUP (self));
-  g_return_if_fail (CLUTTER_IS_ACTOR (actor));
-  g_return_if_fail (sibling == NULL || CLUTTER_IS_ACTOR (sibling));
-
-  if (actor == sibling)
-    return;
-
-  clutter_container_lower_child (CLUTTER_CONTAINER (self), actor, sibling);
-}
-
-/**
- * clutter_group_sort_depth_order:
- * @self: A #ClutterGroup
- *
- * Sorts a #ClutterGroup's children by their depth value.
- * This function should not be used by applications.
- *
- * Deprecated: 0.6: Use clutter_container_sort_depth_order() instead.
- */
-void
-clutter_group_sort_depth_order (ClutterGroup *self)
-{
-  g_return_if_fail (CLUTTER_IS_GROUP (self));
-
-  clutter_container_sort_depth_order (CLUTTER_CONTAINER (self));
-}
