@@ -89,12 +89,13 @@ struct _ClutterStagePrivate
 
   GQueue             *event_queue;
 
-  guint redraw_pending    : 1;
-  guint is_fullscreen     : 1;
-  guint is_offscreen      : 1;
-  guint is_cursor_visible : 1;
-  guint is_user_resizable : 1;
-  guint use_fog           : 1;
+  guint redraw_pending         : 1;
+  guint is_fullscreen          : 1;
+  guint is_offscreen           : 1;
+  guint is_cursor_visible      : 1;
+  guint is_user_resizable      : 1;
+  guint use_fog                : 1;
+  guint throttle_motion_events : 1;
 };
 
 enum
@@ -321,6 +322,10 @@ clutter_stage_show (ClutterActor *self)
 
   CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->show (self);
 
+  /* Possibly do an allocation run so that the stage will have the
+     right size before we map it */
+  _clutter_stage_maybe_relayout (self);
+
   g_assert (priv->impl != NULL);
   impl = CLUTTER_STAGE_WINDOW (priv->impl);
   CLUTTER_STAGE_WINDOW_GET_IFACE (impl)->show (impl, TRUE);
@@ -461,7 +466,8 @@ _clutter_stage_process_queued_events (ClutterStage *stage)
       next_event = l->next ? l->next->data : NULL;
 
       /* Skip consecutive motion events */
-      if (next_event &&
+      if (priv->throttle_motion_events &&
+          next_event &&
 	  event->type == CLUTTER_MOTION &&
 	  (next_event->type == CLUTTER_MOTION ||
 	   next_event->type == CLUTTER_LEAVE))
@@ -791,6 +797,10 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * ClutterStage:offscreen:
    *
    * Whether the stage should be rendered in an offscreen buffer.
+   *
+   * <warning><para>Not every backend supports redirecting the
+   * stage to an offscreen buffer. This property might not work
+   * and it might be deprecated at any later date.</para></warning>
    */
   pspec = g_param_spec_boolean ("offscreen",
                                 "Offscreen",
@@ -1011,11 +1021,12 @@ clutter_stage_init (ClutterStage *self)
 
   priv->event_queue = g_queue_new ();
 
-  priv->is_offscreen      = FALSE;
-  priv->is_fullscreen     = FALSE;
-  priv->is_user_resizable = FALSE;
-  priv->is_cursor_visible = TRUE;
-  priv->use_fog           = FALSE;
+  priv->is_offscreen           = FALSE;
+  priv->is_fullscreen          = FALSE;
+  priv->is_user_resizable      = FALSE;
+  priv->is_cursor_visible      = TRUE;
+  priv->use_fog                = FALSE;
+  priv->throttle_motion_events = TRUE;
 
   priv->color = default_stage_color;
 
@@ -1366,10 +1377,9 @@ clutter_stage_read_pixels (ClutterStage *stage,
                            gint          height)
 {
   guchar *pixels;
-  guchar *temprow;
   GLint   viewport[4];
   gint    rowstride;
-  gint    stage_x, stage_y, stage_width, stage_height;
+  gint    stage_width, stage_height;
 
   g_return_val_if_fail (CLUTTER_IS_STAGE (stage), NULL);
 
@@ -1383,8 +1393,6 @@ clutter_stage_read_pixels (ClutterStage *stage,
   clutter_actor_paint (CLUTTER_ACTOR (stage));
 
   glGetIntegerv (GL_VIEWPORT, viewport);
-  stage_x      = viewport[0];
-  stage_y      = viewport[1];
   stage_width  = viewport[2];
   stage_height = viewport[3];
 
@@ -1397,42 +1405,11 @@ clutter_stage_read_pixels (ClutterStage *stage,
   rowstride = width * 4;
 
   pixels  = g_malloc (height * rowstride);
-  temprow = g_malloc (rowstride);
 
-  /* Setup the pixel store parameters that may have been changed by
-     Cogl */
-  glPixelStorei (GL_PACK_ALIGNMENT, 4);
-#ifdef HAVE_COGL_GL
-  glPixelStorei (GL_PACK_ROW_LENGTH, 0);
-  glPixelStorei (GL_PACK_SKIP_PIXELS, 0);
-  glPixelStorei (GL_PACK_SKIP_ROWS, 0);
-#endif /* HAVE_COGL_GL */
-
-  /* The y co-ordinate should be given in OpenGL's coordinate system
-     so 0 is the bottom row */
-  y = stage_height - y - height;
-
-  glFinish ();
-
-  /* check whether we need to read into a smaller temporary buffer */
-  glReadPixels (x, y, width, height, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
-
-  /* vertically flip the buffer in-place */
-  for (y = 0; y < height / 2; y++)
-    {
-      if (y != height - y - 1) /* skip center row */
-        {
-          memcpy (temprow,
-                  pixels + y * rowstride, rowstride);
-          memcpy (pixels + y * rowstride,
-                  pixels + (height - y - 1) * rowstride, rowstride);
-          memcpy (pixels + (height - y - 1) * rowstride,
-                  temprow,
-                  rowstride);
-        }
-    }
-
-  g_free (temprow);
+  cogl_read_pixels (x, y, width, height,
+                    COGL_READ_PIXELS_COLOR_BUFFER,
+                    COGL_PIXEL_FORMAT_RGBA_8888,
+                    pixels);
 
   return pixels;
 }
@@ -2085,3 +2062,52 @@ _clutter_stage_get_default_window (void)
 
   return _clutter_stage_get_window (CLUTTER_STAGE (stage));
 }
+
+/**
+ * clutter_stage_set_throttle_motion_events:
+ * @stage: a #ClutterStage
+ * @throttle: %TRUE to throttle motion events
+ *
+ * Sets whether motion events received between redraws should
+ * be throttled or not. If motion events are throttled, those
+ * events received by the windowing system between redraws will
+ * be compressed so that only the last event will be propagated
+ * to the @stage and its actors.
+ *
+ * This function should only be used if you want to have all
+ * the motion events delivered to your application code.
+ *
+ * Since: 1.0
+ */
+void
+clutter_stage_set_throttle_motion_events (ClutterStage *stage,
+                                          gboolean      throttle)
+{
+  ClutterStagePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  priv = stage->priv;
+
+  if (priv->throttle_motion_events != throttle)
+    priv->throttle_motion_events = throttle;
+}
+
+/**
+ * clutter_stage_get_throttle_motion_events:
+ * @stage: a #ClutterStage
+ *
+ * Retrieves the value set with clutter_stage_set_throttle_motion_events()
+ *
+ * Return value: %TRUE if the motion events are being throttled,
+ *   and %FALSE otherwise
+ *
+ * Since: 1.0
+ */
+gboolean
+clutter_stage_get_throttle_motion_events (ClutterStage *stage)
+{
+  g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
+
+  return stage->priv->throttle_motion_events;
+}
