@@ -77,8 +77,9 @@ typedef struct _ClutterTextureAsyncData ClutterTextureAsyncData;
 
 struct _ClutterTexturePrivate
 {
-  gfloat                       width;
-  gfloat                       height;
+  gint                         image_width;
+  gint                         image_height;
+
   CoglHandle                   material;
   gboolean                     no_slice;
 
@@ -118,12 +119,17 @@ struct _ClutterTextureAsyncData
   /* Source ID of the idle handler for loading. If this is zero then
      the data is being loaded in a thread from the thread pool. Once
      the thread is finished it will be converted to idle load handler
-     and load_idle will be nonzero. If load_idle is nonzero then the
-     rest of the load can safely be aborted by just removing the
-     source, otherwise the abort flag needs to be set and the data
-     should be disowned */
+     and load_idle will be nonzero. If load_idle is nonzero, and
+     upload_queued is FALSE then the rest of the load can safely be
+     aborted by just removing the source, otherwise the abort flag
+     needs to be set and the data should be disowned */
   guint           load_idle;
 
+  /* Set when the texture is queued for GPU upload, used to determine
+   * what to do with the texture data when load_idle is zero.
+  */
+  gboolean        upload_queued;  
+
   gchar          *load_filename;
   CoglHandle      load_bitmap;
   GError         *load_error;
@@ -159,6 +165,9 @@ enum
 static int texture_signals[LAST_SIGNAL] = { 0 };
 
 static GThreadPool *async_thread_pool = NULL;
+static guint        repaint_upload_func = 0;
+static GList       *upload_list = NULL;
+static GStaticMutex upload_list_mutex = G_STATIC_MUTEX_INIT;
 
 static void
 texture_fbo_free_resources (ClutterTexture *texture);
@@ -293,8 +302,8 @@ clutter_texture_realize (ClutterActor *actor)
       if (priv->no_slice)
         flags |= COGL_TEXTURE_NO_SLICING;
 
-      tex = cogl_texture_new_with_size (priv->width,
-                                        priv->height,
+      tex = cogl_texture_new_with_size (priv->image_width,
+                                        priv->image_height,
                                         flags,
                                         COGL_PIXEL_FORMAT_RGBA_8888_PRE);
 
@@ -313,7 +322,8 @@ clutter_texture_realize (ClutterActor *actor)
           return;
         }
 
-      clutter_actor_set_size (actor, priv->width, priv->height);
+      clutter_actor_set_size (actor, priv->image_width, priv->image_height);
+
       return;
     }
 
@@ -359,14 +369,15 @@ clutter_texture_get_preferred_width (ClutterActor *self,
         {
           if (!priv->keep_aspect_ratio ||
               for_height < 0 ||
-              priv->height <= 0)
+              priv->image_height <= 0)
             {
-              *natural_width_p = priv->width;
+              *natural_width_p = priv->image_width;
             }
           else
             {
               /* Set the natural width so as to preserve the aspect ratio */
-              gfloat ratio = priv->width / priv->height;
+              gfloat ratio = (gfloat) priv->image_width
+                           / (gfloat) priv->image_height;
 
               *natural_width_p = ratio * for_height;
             }
@@ -398,14 +409,15 @@ clutter_texture_get_preferred_height (ClutterActor *self,
         {
           if (!priv->keep_aspect_ratio ||
               for_width < 0 ||
-              priv->width <= 0)
+              priv->image_width <= 0)
             {
-              *natural_height_p = priv->height;
+              *natural_height_p = priv->image_height;
             }
           else
             {
               /* Set the natural height so as to preserve the aspect ratio */
-              gfloat ratio = priv->height / priv->width;
+              gfloat ratio = (gfloat) priv->image_height
+                           / (gfloat) priv->image_width;
 
               *natural_height_p = ratio * for_width;
             }
@@ -557,8 +569,10 @@ clutter_texture_paint (ClutterActor *self)
 	  /* Use a projection matrix that makes the actor appear as it
 	     would if it was rendered at its normal screen location */
 	  clutter_texture_set_fbo_projection (self);
+
 	  /* Reset the viewport to the size of the FBO */
-	  cogl_viewport (priv->width, priv->height);
+	  cogl_viewport (priv->image_width, priv->image_height);
+
 	  /* Reapply the source's parent transformations */
 	  if ((source_parent = clutter_actor_get_parent (priv->fbo_source)))
 	    _clutter_actor_apply_modelview_transform_recursive (source_parent,
@@ -596,13 +610,6 @@ clutter_texture_paint (ClutterActor *self)
         clutter_shader_set_is_enabled (shader, TRUE);
     }
 
-  /* A clone may need to fire above if were a TFP/FBO but not visible.
-   * Ultimatly needs some reworking with maybe an extra prepare_paint
-   * method or some such.
-  */
-  if (CLUTTER_PRIVATE_FLAGS(self) & CLUTTER_TEXTURE_IN_CLONE_PAINT)
-    return;
-
   CLUTTER_NOTE (PAINT,
                 "painting texture '%s'",
 		clutter_actor_get_name (self) ? clutter_actor_get_name (self)
@@ -618,13 +625,13 @@ clutter_texture_paint (ClutterActor *self)
 		box.x1, box.y1, box.x2, box.y2,
 		clutter_actor_get_opacity (self));
 
-  if (priv->repeat_x && priv->width > 0)
-    t_w = (box.x2 - box.x1) / priv->width;
+  if (priv->repeat_x && priv->image_width > 0)
+    t_w = (box.x2 - box.x1) / (gfloat) priv->image_width;
   else
     t_w = 1.0;
 
-  if (priv->repeat_y && priv->height > 0)
-    t_h = (box.y2 - box.y1) / priv->height;
+  if (priv->repeat_y && priv->image_height > 0)
+    t_h = (box.y2 - box.y1) / (gfloat) priv->image_height;
   else
     t_h = 1.0;
 
@@ -641,8 +648,8 @@ clutter_texture_async_data_free (ClutterTextureAsyncData *data)
 {
   /* This function should only be called either from the main thread
      once it is known that the load thread has completed or from the
-     load thread itself if the abort flag is true (in which case the
-     main thread has disowned the data) */
+     load thread/upload function itself if the abort flag is true (in
+     which case the main thread has disowned the data) */
 
   if (data->load_filename)
     g_free (data->load_filename);
@@ -696,7 +703,8 @@ clutter_texture_async_load_cancel (ClutterTexture *texture)
       else
         {
           /* Otherwise we need to tell the thread to abort and disown
-             the data */
+             the data, if the data has been loaded and decoded the data
+             is now waiting for a master clock iteration to be repainted */
           priv->async_data->abort = TRUE;
 
           if (mutex)
@@ -1040,7 +1048,7 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 			   "Load files inside a thread to avoid blocking when "
                            "loading images.",
 			   FALSE,
-			   CLUTTER_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
+			   CLUTTER_PARAM_WRITABLE));
 
 
   /**
@@ -1058,7 +1066,7 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 			   "Decode image data files inside a thread to reduce "
                            "blocking when loading images.",
 			   FALSE,
-			   CLUTTER_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
+			   CLUTTER_PARAM_WRITABLE));
 
 
 
@@ -1078,9 +1086,10 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterTextureClass, size_change),
 		  NULL, NULL,
-		  clutter_marshal_VOID__FLOAT_FLOAT,
-		  G_TYPE_NONE,
-		  2, G_TYPE_FLOAT, G_TYPE_FLOAT);
+		  clutter_marshal_VOID__INT_INT,
+		  G_TYPE_NONE, 2,
+                  G_TYPE_INT,
+                  G_TYPE_INT);
   /**
    * ClutterTexture::pixbuf-change:
    * @texture: the texture which received the signal
@@ -1367,8 +1376,8 @@ clutter_texture_set_cogl_texture (ClutterTexture  *texture,
 				  CoglHandle       cogl_tex)
 {
   ClutterTexturePrivate  *priv;
-  gboolean                size_change;
-  guint                   width, height;
+  gboolean size_changed;
+  guint width, height;
 
   g_return_if_fail (CLUTTER_IS_TEXTURE (texture));
   g_return_if_fail (cogl_is_texture (cogl_tex));
@@ -1410,19 +1419,19 @@ clutter_texture_set_cogl_texture (ClutterTexture  *texture,
      safely release the reference we claimed above */
   cogl_handle_unref (cogl_tex);
 
-  size_change      = width != priv->width || height != priv->height;
-  priv->width      = width;
-  priv->height     = height;
+  size_changed = (width != priv->image_width || height != priv->image_height);
+  priv->image_width = width;
+  priv->image_height = height;
 
-  CLUTTER_NOTE (TEXTURE, "set size %.2fx%.2f\n",
-		priv->width,
-		priv->height);
+  CLUTTER_NOTE (TEXTURE, "set size (w:%d, h:%d)",
+		priv->image_width,
+		priv->image_height);
 
-  if (size_change)
+  if (size_changed)
     {
       g_signal_emit (texture, texture_signals[SIZE_CHANGE], 0,
-                     priv->width,
-                     priv->height);
+                     priv->image_width,
+                     priv->image_height);
 
       if (priv->sync_actor_size)
         clutter_actor_queue_relayout (CLUTTER_ACTOR (texture));
@@ -1651,10 +1660,11 @@ clutter_texture_async_load_complete (ClutterTexture *self,
                                              flags,
                                              COGL_PIXEL_FORMAT_ANY);
       clutter_texture_set_cogl_texture (self, handle);
+
       if (priv->load_size_async)
         {
           g_signal_emit (self, texture_signals[SIZE_CHANGE], 0,
-                         cogl_texture_get_width(handle),
+                         cogl_texture_get_width (handle),
                          cogl_texture_get_height (handle));
         }
 
@@ -1674,6 +1684,12 @@ clutter_texture_thread_idle_func (gpointer user_data)
   /* Grab the mutex so we can be sure the thread has unlocked it
      before we destroy it */
   g_mutex_lock (data->mutex);
+  if (data->abort)
+    {
+      g_mutex_unlock (data->mutex);
+      clutter_texture_async_data_free (data);
+      return FALSE;
+    }
   g_mutex_unlock (data->mutex);
 
   clutter_texture_async_load_complete (data->texture, data->load_bitmap,
@@ -1684,6 +1700,44 @@ clutter_texture_thread_idle_func (gpointer user_data)
   return FALSE;
 }
 
+static gboolean
+texture_repaint_upload_func (gpointer user_data)
+{
+  gulong start_time;
+
+  g_static_mutex_lock (&upload_list_mutex);
+
+  if (upload_list)
+    {
+      start_time = clutter_get_timestamp ();
+
+      /* continue uploading textures as long as we havent spent more
+       * then 5ms doing so this stage redraw cycle.
+       */
+      do
+        {
+          ClutterTextureAsyncData *data = upload_list->data;
+
+          clutter_texture_thread_idle_func (data);
+
+          upload_list = g_list_remove (upload_list, data);
+        }
+      while (upload_list && clutter_get_timestamp () < start_time + 5 * 1000);
+    }
+
+  if (upload_list)
+    {
+      ClutterMasterClock *master_clock;
+
+      master_clock = _clutter_master_clock_get_default ();
+      _clutter_master_clock_ensure_next_iteration (master_clock);
+    }
+
+  g_static_mutex_unlock (&upload_list_mutex);
+
+  return TRUE;
+}
+
 static void
 clutter_texture_thread_func (gpointer user_data, gpointer pool_data)
 {
@@ -1718,19 +1772,32 @@ clutter_texture_thread_func (gpointer user_data, gpointer pool_data)
     }
   else
     {
+      ClutterMasterClock *master_clock = _clutter_master_clock_get_default ();
+
       /* Make sure we give the image to GL in the main thread, where we
        * hold the main Clutter lock. Once load_idle is non-NULL then the
        * main thread is guaranteed not to set the abort flag. It can't
        * set it while we're holding the mutex so we can safely start the
        * idle handler now without the possibility of calling the
        * callback after it is aborted */
-      data->load_idle =
-        clutter_threads_add_idle_full (G_PRIORITY_LOW, clutter_texture_thread_idle_func, data, NULL);
+      g_static_mutex_lock (&upload_list_mutex);
+
+      if (repaint_upload_func == 0)
+        {
+          repaint_upload_func =
+            clutter_threads_add_repaint_func (texture_repaint_upload_func,
+                                              NULL, NULL);
+        }
+
+      upload_list = g_list_append (upload_list, data);
+      data->upload_queued = TRUE;
+
+      g_static_mutex_unlock (&upload_list_mutex);
 
       g_mutex_unlock (data->mutex);
-    }
 
-  return;
+      _clutter_master_clock_ensure_next_iteration (master_clock);
+    }
 }
 
 static gboolean
@@ -1812,8 +1879,8 @@ clutter_texture_async_load (ClutterTexture *self,
     }
   else
     {
-      priv->width = width;
-      priv->height = height;
+      priv->image_width = width;
+      priv->image_height = height;
     }
 
   clutter_texture_async_load_cancel (self);
@@ -2101,10 +2168,10 @@ clutter_texture_get_base_size (ClutterTexture *texture,
   g_return_if_fail (CLUTTER_IS_TEXTURE (texture));
 
   if (width)
-    *width = texture->priv->width;
+    *width = texture->priv->image_width;
 
   if (height)
-    *height = texture->priv->height;
+    *height = texture->priv->image_height;
 }
 
 /**
@@ -2225,7 +2292,7 @@ on_fbo_source_size_change (GObject          *object,
 
   clutter_actor_get_transformed_size (priv->fbo_source, &w, &h);
 
-  if (w != priv->width || h != priv->height)
+  if (w != priv->image_width || h != priv->image_height)
     {
       CoglTextureFlags flags = COGL_TEXTURE_NONE;
       CoglHandle tex;
@@ -2236,13 +2303,13 @@ on_fbo_source_size_change (GObject          *object,
 
       texture_free_gl_resources (texture);
 
-      priv->width = w;
-      priv->height = h;
+      priv->image_width = w;
+      priv->image_height = h;
 
       flags |= COGL_TEXTURE_NO_SLICING;
 
-      tex = cogl_texture_new_with_size (MAX (priv->width, 1),
-                                        MAX (priv->height, 1),
+      tex = cogl_texture_new_with_size (MAX (priv->image_width, 1),
+                                        MAX (priv->image_height, 1),
                                         flags,
                                         COGL_PIXEL_FORMAT_RGBA_8888_PRE);
 
@@ -2432,10 +2499,12 @@ clutter_texture_new_from_actor (ClutterActor *actor)
                     G_CALLBACK(on_fbo_parent_change),
                     texture);
 
-  priv->width = w;
-  priv->height = h;
+  priv->image_width = w;
+  priv->image_height = h;
 
-  clutter_actor_set_size (CLUTTER_ACTOR (texture), priv->width, priv->height);
+  clutter_actor_set_size (CLUTTER_ACTOR (texture),
+                          priv->image_width,
+                          priv->image_height);
 
   return CLUTTER_ACTOR (texture);
 }
