@@ -1,11 +1,9 @@
 /*
- * Clutter COGL
+ * Cogl
  *
- * A basic GL/GLES Abstraction/Utility Layer
+ * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Authored By Matthew Allum  <mallum@openedhand.com>
- *
- * Copyright (C) 2007 OpenedHand
+ * Copyright (C) 2007,2008,2009 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,14 +24,16 @@
 #ifndef __COGL_FBO_H
 #define __COGL_FBO_H
 
+#include "cogl-handle.h"
+
 typedef struct
 {
-  guint  ref_count;
+  CoglHandleObject _parent;
   int    width;
   int    height;
   GLuint gl_handle;
   GLuint gl_stencil_handle;
-  
+
 } CoglFbo;
 
 #endif /* __COGL_FBO_H */
